/* WinCompat.c
 * This file is part of the LaBrea package
 *
 * Copyright (C) 2001-2003 Tom Liston <tliston@premmag.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


/*           VERY, VERY IMPORTANT NOTE:             */
/* THIS IS NOT A "GENERAL REPLACEMENT" FOR LIBNET   */
/* THIS IS ONLY CODED TO ALLOW THE WIN32 VERSION OF */
/* LABREA TO FUNCTION WITHOUT NEEDING LIBNETNT BY   */
/* USING THE PACKET.DLL FUNCTIONS AS A REPLACEMENT. */
/* ANY ATTEMPT TO USE THIS TO DO *ANYTHING ELSE*    */
/* WILL MOST LIKELY END IN DISMAL FAILURE.          */
/*            YOU HAVE BEEN WARNED!                 */

#ifdef WIN32
#include "includes.h"
#include <ws2tcpip.h>
#include <iprtrmib.h>

/* IP Stuff */
#define LEN_OFFSET    2
#define TOS_OFFSET    1
#define IPID_OFFSET   4
#define FRAG_OFFSET   6
#define TTL_OFFSET    8
#define IPPROT_OFFSET 9
#define SADDR_OFFSET  12
#define DADDR_OFFSET  16
#define IP_CHECKSUM   10

/* TCP Stuff */
#define SPORT_OFFSET  0
#define DPORT_OFFSET  2
#define SEQ_OFFSET    4
#define ACK_OFFSET    8
#define FLAGS_OFFSET  13
#define WIN_OFFSET    14
#define URG_OFFSET    18
#define TCP_CHECKSUM  16

/* ICMP Stuff */
#define ICMP_CHECKSUM 2

unsigned long getnetmask(char *netstr);

#define MAX_NUM_ADAPTER 10

static __inline int in_cksum(u_short *addr, int len) {
  int sum;
  int nleft;
  u_short ans;
  u_short *w;

  sum = 0;
  ans = 0;
  nleft = len;
  w = addr;

  while(nleft > 1) {
    sum += *w++;
    nleft -= 2;
  }
  if(nleft == 1) {
    *(u_char *)(&ans) = *(u_char *)w;
    sum += ans;
  }
  return(sum);
}

int libnet_seed_prand(void) {
  srand( (unsigned)time( NULL ) );
  return 1;
}

unsigned long libnet_get_prand(int dummy) {
  unsigned long rv;

  rv = (unsigned)rand() << 16;
  rv += (unsigned)rand();
  return(rv);
}

unsigned long libnet_name_resolve(unsigned char *hostname, unsigned short dummy) {
  return(inet_addr((const char*)hostname));
}


int countem(int number) {
  switch(number) {
    case 255:
      return 8;
    case 254:
      return 7;
    case 252:
      return 6;
    case 248:
      return 5;
    case 240:
      return 4;
    case 224:
      return 3;
    case 192:
      return 2;
    case 128:
      return 1;
    default:
      return 0;
  }
}



BOOL GetIPStuff() {
  WSADATA  wsaData;
  char     szHostname[100];
  HOSTENT *pHostEnt;
  char address_buffer[4096];
  SOCKET sAddr;
  INTERFACE_INFO *if_info;
  DWORD bytes_back;
  union sockaddr_gen *sockaddr;
  struct sockaddr_in sockaddr_in;
  struct sockaddr_in sockaddr_in2;
  int count, offset = 0, notfound = 1;
  extern char ipaddr[20];
  extern int testflag;

  if(WSAStartup(0x0202, &wsaData)) {
    printf("Error: Failed to open WinSock2 library, error code %u.\n", WSAGetLastError());
    return 0;
  }
  gethostname(szHostname, sizeof(szHostname));
  pHostEnt = gethostbyname(szHostname);
  if(pHostEnt->h_addr_list[0])
    memcpy ( &sockaddr_in2.sin_addr.s_addr, pHostEnt->h_addr_list[0], pHostEnt->h_length);
  sAddr = WSASocket(PF_INET, SOCK_RAW, AF_INET, 0, 0, 0);
  if(sAddr == INVALID_SOCKET) {
    printf("Error: Failed to open the query socket, error code %u.\n", WSAGetLastError());
    return 0;
  }
  memset(address_buffer, 0, 4096);
  if(WSAIoctl(sAddr, SIO_GET_INTERFACE_LIST, NULL, 0, (void *)address_buffer, 4096, &bytes_back, 0, 0)) {
    printf("Error: Failed to query WinSock2, error code %u.\n", WSAGetLastError());
    return 0;
  }
  count = bytes_back / sizeof(INTERFACE_INFO);
  closesocket(sAddr);
  WSACleanup();
  while(notfound) {
    if_info = ((INTERFACE_INFO *) address_buffer) + offset;
    sockaddr = (union sockaddr_gen*) &(if_info->iiNetmask);
    sockaddr_in = sockaddr->AddressIn;
    count = countem(sockaddr_in.sin_addr.S_un.S_un_b.s_b1);
    count += countem(sockaddr_in.sin_addr.S_un.S_un_b.s_b2);
    count += countem(sockaddr_in.sin_addr.S_un.S_un_b.s_b3);
    count += countem(sockaddr_in.sin_addr.S_un.S_un_b.s_b4);
    sockaddr = (union sockaddr_gen*) &(if_info->iiAddress);
    sockaddr_in = sockaddr->AddressIn;
    if(sockaddr_in.sin_addr.S_un.S_addr == sockaddr_in2.sin_addr.S_un.S_addr)
      notfound = 0;
    offset++;
    if(offset == count)
      return 0;
  }
  sprintf(ipaddr, "%d.%d.%d.%d/%d",
          sockaddr_in.sin_addr.S_un.S_un_b.s_b1,
          sockaddr_in.sin_addr.S_un.S_un_b.s_b2,
          sockaddr_in.sin_addr.S_un.S_un_b.s_b3,
          sockaddr_in.sin_addr.S_un.S_un_b.s_b4,
          count);
  return 1;
}


BOOL GetAMAC(int index) {
  NCB Ncb;
  UCHAR uRetCode;
  LANA_ENUM lenum;
  ASTAT Adapter;
  int i = index - 1;
  extern int testflag;
  extern u_char MACAddr[6];

  memset( &Ncb, 0, sizeof(Ncb) );
  Ncb.ncb_command = NCBENUM;
  Ncb.ncb_buffer = (UCHAR *)&lenum;
  Ncb.ncb_length = sizeof(lenum);
  uRetCode = Netbios( &Ncb );
  if(uRetCode != 0)
    return 0;
  memset( &Ncb, 0, sizeof(Ncb) );
  Ncb.ncb_command = NCBRESET;
  Ncb.ncb_lana_num = lenum.lana[i];
  uRetCode = Netbios(&Ncb);
  if(uRetCode != 0)
    return 0;
  memset( &Ncb, 0, sizeof (Ncb) );
  Ncb.ncb_command = NCBASTAT;
  Ncb.ncb_lana_num = lenum.lana[i];
  strcpy( Ncb.ncb_callname,  "*               " );
  Ncb.ncb_buffer = (char *) &Adapter;
  Ncb.ncb_length = sizeof(Adapter);
  uRetCode = Netbios( &Ncb );
  if(uRetCode == 0) {
    if(testflag) {
      printf( "The MAC address of interface %d is: %02x:%02x:%02x:%02x:%02x:%02x\n",
              index,
              Adapter.adapt.adapter_address[0],
              Adapter.adapt.adapter_address[1],
              Adapter.adapt.adapter_address[2],
              Adapter.adapt.adapter_address[3],
              Adapter.adapt.adapter_address[4],
              Adapter.adapt.adapter_address[5] );
    }
    for(i = 0; i < 6; i++)
      MACAddr[i] = Adapter.adapt.adapter_address[i];
    return 1;
  }
  return 0;
}


void* GetAdapterFromList(int index) {
  DWORD dwVersion;
  DWORD dwWindowsMajorVersion;
  char* Adapter95;
  WCHAR* Adapter;
  int i;
  PTSTR device;
  ULONG devlen = 4096;

  device = malloc(devlen);
  if(!device)
    CleanExit(0);
  PacketGetAdapterNames(device, &devlen);
  dwVersion=GetVersion();
  dwWindowsMajorVersion = (DWORD)(LOBYTE(LOWORD(dwVersion)));
  if(dwVersion >= 0x80000000 && dwWindowsMajorVersion >= 4) {      // Windows '95
    Adapter95 = device;
    for(i = 0; i < (index-1); i++) {
      while(*Adapter95++ != 0);
      if(*Adapter95 == 0)
        return NULL;
    }
    return Adapter95;
  } else {
    Adapter = (WCHAR*)device;
    for(i = 0; i < (index - 1); i++) {
      while(*Adapter++ != 0);
      if(*Adapter == 0)
        return NULL;
    }
    return Adapter;
  }
}


void PrintDeviceList() {
  DWORD dwVersion;
  DWORD dwWindowsMajorVersion;
  const WCHAR* t;
  const char* t95;
  int i = 0;
  int DescPos = 0;
  char *Desc;
  int n = 1;
  PTSTR device;
  ULONG devlen = 4096;

  device = malloc(devlen);
  if(!device)
    CleanExit(0);
  PacketGetAdapterNames(device, &devlen);
  dwVersion = GetVersion();
  dwWindowsMajorVersion =  (DWORD)(LOBYTE(LOWORD(dwVersion)));
  if(dwVersion >= 0x80000000 && dwWindowsMajorVersion >= 4) {      // Windows '95
    t95 = (char*)device;
    while(*(t95+DescPos) != 0 || *(t95 + DescPos-1) != 0) {
      DescPos++;
    }
    Desc = (char*)t95 + DescPos+1;
    printf("%d. ", n++);
    while(!(t95[i] == 0 && t95[i-1] == 0)) {
      if(t95[i] == 0) {
        putchar(' ');
        putchar('(');
        while(*Desc != 0) {
          putchar(*Desc);
          Desc++;
        }
        Desc++;
        putchar(')');
        putchar('\n');
      } else
        putchar(t95[i]);
      if((t95[i] == 0) && (t95[i+1] != 0)) {
        printf("%d. ",n++);
      }
      i++;
    }
    putchar('\n');
  } else {   //Windows NT
    t = (WCHAR*)device;
    while(*(t + DescPos) != 0 || *(t + (DescPos - 1)) != 0) {
      DescPos++;
    }
    DescPos <<= 1;
    Desc = (char*)t + DescPos + 2;
    printf("%d. ", n++);
    while(!((t[i] == 0) && (t[i - 1] == 0))) {
      if(t[i] == 0) {
        putchar(' ');
        putchar('(');
        while(*Desc != 0) {
          putchar(*Desc);
          Desc++;
        }
        Desc++;
        putchar(')');
        putchar('\n');
      } else
        putchar(t[i]);
      if((t[i] == 0) && (t[i + 1] != 0))
        printf("%d. ", n++);
      i++;
    }
    putchar('\n');
  }
  free(device);
}


int libnet_close_link_interface(struct libnet_link_int *l) {
  PacketFreePacket(l->lpOutPacket);
  PacketCloseAdapter(l->lpAdapter);
  MYFREE(l->mypacket);
  MYFREE(l);
  return 1;
}

struct libnet_link_int *libnet_open_link_interface(char *device, char *dummy) {
  struct libnet_link_int *rv;
  LPADAPTER lpa;

  lpa = PacketOpenAdapter(device);
  if(!lpa || (lpa->hFile == INVALID_HANDLE_VALUE))
    return NULL;
  if((rv = malloc(sizeof(struct libnet_link_int))) == NULL)
    return NULL;
  rv->linkoffset = 14;
  rv->lpAdapter = lpa;
  if((rv->lpOutPacket = PacketAllocatePacket()) == NULL) {
    MYFREE(rv);
    rv = NULL;
  }
  return rv;
}

int libnet_do_checksum(u_char *buf, int protocol, int len) {
  int sum;

  switch(protocol) {
    case IPPROTO_TCP:
      /* tcp checksum */
      *((short *)(buf + TCP_CHECKSUM + 20)) = 0;
      sum = in_cksum((u_short *)(buf + SADDR_OFFSET), 8);
      sum += ntohs(6 + 20);
      sum += in_cksum((u_short *)(buf + SPORT_OFFSET + 20), 20);
      *((short *)(buf + TCP_CHECKSUM + 20)) = CKSUM_CARRY(sum);
      break;
    case IPPROTO_IP:
      /* ip checksum */
      *((short *)(buf + IP_CHECKSUM)) = 0;
      sum = in_cksum((u_short *)(buf), 20);
      *((short *)(buf + IP_CHECKSUM)) = CKSUM_CARRY(sum);
      break;
    case IPPROTO_ICMP:
      /* ip checksum */
      *((short *)(buf + ICMP_CHECKSUM + 20)) = 0;
      sum = in_cksum((u_short *)(buf), 20 + len);
      *((short *)(buf + ICMP_CHECKSUM + 20)) = CKSUM_CARRY(sum);
      break;
  }
  return 1;
}

unsigned long getnetmask(char *netstr) {
  int netbits, i;
  unsigned long net = 0;

  netbits = atol(netstr);
  if((netbits <= 0) || (netbits > 32))
    netbits = 32;
  for(i = 0; i < 32; i++) {
    net <<= 1;
    if(i < netbits)
      net |= 1;
  }
  return htonl(net);
}


int libnet_build_ip(u_short len, u_char tos, u_short id, u_short frag,
                    u_char ttl, u_char prot, u_long saddr, u_long daddr,
                    const u_char *payload, int payload_s, u_char *buf) {
  *((u_short*)(buf+ LEN_OFFSET)) = htons((u_short)(len + 20));
  *((u_char*)(buf + TOS_OFFSET)) = tos;
  *((u_short*)(buf + IPID_OFFSET)) = htons(id);
  *((u_short*)(buf + FRAG_OFFSET)) = htons(frag);
  *((u_char*)(buf + TTL_OFFSET)) = ttl;
  *((u_char*)(buf + IPPROT_OFFSET)) = prot;
  *((u_long*)(buf + SADDR_OFFSET)) = saddr;
  *((u_long*)(buf + DADDR_OFFSET)) = daddr;
  return 1;
}

int libnet_build_tcp(u_short sport, u_short dport, u_long seq,
                     u_long ack, u_char control, u_short win, u_short urg,
                     const u_char *payload, int payload_s, u_char *buf) {
  *((u_short*)(buf + SPORT_OFFSET)) = htons(sport);
  *((u_short*)(buf + DPORT_OFFSET)) = htons(dport);
  *((u_long*)(buf + SEQ_OFFSET)) = htonl(seq);
  *((u_long*)(buf + ACK_OFFSET)) = htonl(ack);
  *((u_char*)(buf + 12)) = 80;
  *((u_char*)(buf + FLAGS_OFFSET)) = control;
  *((u_short*)(buf + WIN_OFFSET)) = htons(win);
  *((u_short*)(buf + URG_OFFSET)) = urg;
  return 1;
}

int libnet_write_link_layer(struct libnet_link_int *l, const u_char *device,
                            u_char *buf, int len) {
  PacketInitPacket(l->lpOutPacket, (char *)buf, len);
  PacketSendPacket(l->lpAdapter, l->lpOutPacket, 1);
  return 1;
}

#endif



